/*$Id: mg_in.cc,v 26.9 2006/12/04 08:00:15 al Exp $ -*- C++ -*-
 * Copyright (C) 2001 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
//testing=script 2006.10.31
#include "mg_.h"
/*--------------------------------------------------------------------------*/
static C_Comment   dummy_c_comment;
static Cxx_Comment dummy_cxx_comment;
/*--------------------------------------------------------------------------*/
void Parameter::parse(CS& file)
{
  file >> _type >> _code_name >> _comment;
  int here = file.cursor();
  for (;;) {
    ONE_OF 
      || get(file, "name",	     &_user_name)
      || get(file, "alt_name",	     &_alt_name)
      || get(file, "default",	     &_default_val)
      || get(file, "offset",	     &_offset)
      || set(file, "positive",	     &_positive,	true)
      || set(file, "octal",	     &_octal,		true)
      || get(file, "print_test",     &_print_test)
      || get(file, "calc_print_test",&_calc_print_test)
      || get(file, "scale",	     &_scale)
      || get(file, "calculate",	     &_calculate)
      || get(file, "quiet_min",	     &_quiet_min)
      || get(file, "quiet_max",	     &_quiet_max)
      || get(file, "final_default",  &_final_default)
      || get(file, "/*",	     &dummy_c_comment)
      || get(file, "//",	     &dummy_cxx_comment)
      ;
    if (file.skip1b(";,")) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (parameter)");
      break;
    }else if (file.stuck(&here)) {itested();
      file.warn(0, "need ;");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Parameter::print(std::ostream& out)const
{
  out << "\n"
    "      " << type() << " " << code_name() << " \"" << comment() << "\"\n        "
    "name=\"" << user_name() << "\" "
    "alt_name=\"" << alt_name() << "\" "
    "default=\"" << default_val() << "\" "
    "offset=\"" << offset() << "\"\n        ";
  if (positive()) {
    out << "positive ";
  }
  if (octal()) {
    out << "octal ";
  }
  out <<
    "print_test=\"" << print_test() << "\" "
    "calc_print_test=\"" << calc_print_test() << "\"\n        "
    "scale=\"" << scale() << "\" "
    "calculate=\"" << calculate() << "\"\n        "
    "quiet_min=\"" << quiet_min() << "\" "
    "quiet_max=\"" << quiet_max() << "\"\n        "
    "final_default=\"" << final_default() << "\";";
}
/*--------------------------------------------------------------------------*/
void Code_Block::parse(CS& file) 
{
  // skips the code block, delimited by {}
  // sets _begin and _end, making that part of file the string
  // does not copy it.
  // also checks paren balance, so you can have {} inside the block

  int paren = file.skip1("{");
  if (paren == 0) {untested();
    file.warn(0, "need {");
  }else{
  }
  int here = file.cursor();
  _begin = file.tail();
  for (;;) {
    paren -= file.skip1b("])");
    if (paren == 0) {untested();
      file.warn(0, "unbalanced {}[]()");
      break;
    }else{
    }
    _end = file.tail();
    paren -= file.skip1b("}");
    if (paren == 0) {
      while (*(--_end) != '\n' && _end >= _begin) {
      }
      ++_end;
      break;
    }else{
    }
    if (file.stuck(&here)) {untested();
      file.warn(0, "syntax error");
      break;
    }else{
    }
    paren += file.skip1b("{[(");
    file.skip1b(";=");
    std::string foo;
    file >> foo;
  }
}
/*--------------------------------------------------------------------------*/
static void fill_in_default_names(Parameter_List& pl)
{
  for (Parameter_List::const_iterator
       p = pl.begin();
       p != pl.end();
       ++p) {
    (**p).fill_in_default_name();
  }
}
/*--------------------------------------------------------------------------*/
void Parameter_Block::parse(CS& file)
{ 
  int paren = file.skip1b("{");
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "unnamed",		&_unnamed_value)
      || get(file, "override",		&_override)
      || get(file, "raw_parameters",	&_raw)
      || get(file, "calculated_parameters",&_calculated)
      || get(file, "code_pre",		&_code_pre)
      || get(file, "code_mid",		&_code_mid)
      || get(file, "code_post",		&_code_post)
      || get(file, "/*",		&dummy_c_comment)
      || get(file, "//",		&dummy_cxx_comment)
      ;
    paren -= file.skip1b("}");
    if (paren == 0) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Parameter_Block)");
      break;
    }else if (file.stuck(&here)) {itested();
      file.warn(0, "bad Parameter_Code_Block");
      break;
    }else{
    }
  }
  fill_in_default_names(_calculated);
  fill_in_default_names(_raw);
  // but not _override
}
/*--------------------------------------------------------------------------*/
void Parameter_Block::print(std::ostream& out)const
{
  out << "{\n";
  if (unnamed_value() != "") {
    out << "    unnamed " << unnamed_value() << ";\n";
  }
  out <<
    "    override " << override() << "\n"
    "    raw_parameters " << raw() << "\n"
    "    calculated_parameters " << calculated() << "\n"
    "    code_pre {" << code_pre() << "  }\n"
    "    code_mid {" << code_mid() << "  }\n"
    "    code_post {" << code_post() << "  }\n"
    "  }\n";
}
/*--------------------------------------------------------------------------*/
void Eval::parse(CS& file)
{
  file >> _name >> _code;
}
/*--------------------------------------------------------------------------*/
void Eval::print(std::ostream& out)const
{
  out << "  eval " << name() << " {"
      << code() <<
    "  }\n";
}
/*--------------------------------------------------------------------------*/
void Function::parse(CS& file)
{
  file >> _name;
  _name += '(' + file.ctos("{", "(", ")") + ')';
  file >> _code;
}
/*--------------------------------------------------------------------------*/
void Function::print(std::ostream& out)const
{
  out << "  function " << name() << " {"
      << code() <<
    "  }\n";
}
/*--------------------------------------------------------------------------*/
void Element::parse(CS& file)
{
  file >> _dev_type >> _name >> _port_list;
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "eval",	&_eval)
      || get(file, "value",	&_value)
      || get(file, "args",	&_args)
      || get(file, "omit",	&_omit)
      || get(file, "reverse",	&_reverse)
      || get(file, "state",	&_state)
      || get(file, "/*",	&dummy_c_comment)
      || get(file, "//",	&dummy_cxx_comment)
      ;
    if (file.skip1b(";")) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Element)");
      break;
    }else if (file.stuck(&here)) {untested();
      file.warn(0, "need ;");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Element::print(std::ostream& out)const
{
  out << "    " << dev_type() << " " << name() << " " << ports();
  if (eval() != "") {
    out << " eval=\"" << eval() << "\"";
  }
  if (value() != "") {
    out << " value=\"" << value() << "\"";
  }
  if (args() != "") {
    out << " args=\"" << args() << "\"";
  }
  if (omit() != "") {
    out << " omit=\"" << omit() << "\"";
  }
  if (reverse() != "") {
    out << " reverse=\"" << reverse() << "\"";
  }
  if (state() != "") {
    out << " state=\"" << state() << "\"";
  }
  out << ";\n";
}
/*--------------------------------------------------------------------------*/
void Arg::parse(CS& file)
{
  file.skipbl();
  _arg = file.get_to(";");
  file.skip1b(";");
}
/*--------------------------------------------------------------------------*/
void Circuit::parse(CS& file)
{
  int paren = file.skip1b("{");
  get(file, "/*", &dummy_c_comment);
  get(file, "//", &dummy_cxx_comment);
  get(file, "sync", &_sync) && file.skip1b(";");
  (get(file, "ports", &_required_nodes)
   && ((file >> _optional_nodes), file.skip1b(";")))
    || file.warn(0, "need ports");
  get(file, "local_nodes", &_local_nodes) && file.skip1b(";");
  int here = file.cursor();
  do {
    get(file, "args", &_args_list);
  } while (file.more() && !file.stuck(&here));
  for (;;) {
    paren -= file.skip1b("}");
    if (paren == 0) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Circuit)");
      break;
    }else{
    }
    ONE_OF
      || get(file, "/*", &dummy_c_comment)
      || get(file, "//", &dummy_cxx_comment)
      || (file >> _element_list)
      ;
    if (file.stuck(&here)) {untested();
      file.warn(0, "bad Circuit");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Circuit::print(std::ostream& out)const
{
  out << " {\n";
  if (sync()) {
    out << "  sync;\n";
  }
  out <<
    "  ports " << req_nodes() << opt_nodes() << ";\n"
    "  local_nodes " << local_nodes() << "\n"
	       << args_list()
	       << elements() <<
    "}\n";
}
/*--------------------------------------------------------------------------*/
void Model::parse(CS& file)
{
  file >> _name;
  int paren = file.skip1b("{");
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "hide_base",		&_hide_base)
      || get(file, "level",		&_level)
      || get(file, "dev_type",		&_dev_type)
      || get(file, "inherit",		&_inherit)
      || get(file, "public_keys",	&_public_key_list)
      || get(file, "private_keys",	&_private_key_list)
      || get(file, "independent",	&_independent)
      || get(file, "size_dependent",	&_size_dependent)
      || get(file, "temperature_dependent", &_temperature)
      || get(file, "tr_eval",		&_tr_eval)
      || get(file, "validate",		&_validate)
      || get(file, "/*",		&dummy_c_comment)
      || get(file, "//",		&dummy_cxx_comment)
      ;
    paren -= file.skip1b("}");
    if (paren == 0) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Model)");
      break;
    }else if (file.stuck(&here)) {itested();
      file.warn(0, "bad Model");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Model::print(std::ostream& out)const
{
  out << "model " << name() << " {\n";
  if (hide_base()) {
    out << "  hide_base;\n";
  }
  out << 
    "  level = " << level() << ";\n"
    "  dev_type = " << dev_type() << ";\n"
    "  inherit = " << inherit() << ";\n"
    "  public_keys " << public_key_list() << "\n"
    "  private_keys " << private_key_list() << "\n"
    "  independent " << independent() << "\n"
    "  size_dependent " << size_dependent() << "\n"
    "  temperature_dependent " << temperature() << "\n"
    "  tr_eval {" << tr_eval() << "  }\n"
    "  validate {" << validate() << "  }\n"
    "}\n";
}
/*--------------------------------------------------------------------------*/
void Device::parse(CS& file)
{
  file >> _name;
  int paren = file.skip1b("{");
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "parse_name",&_parse_name)
      || get(file, "id_letter",	&_id_letter)
      || get(file, "model_type",&_model_type)
      || get(file, "circuit",	&_circuit)
      || get(file, "tr_probe",	&_probes)
      || get(file, "device",	&_device)
      || get(file, "common", 	&_common)
      || get(file, "tr_eval",	&_tr_eval)
      || get(file, "eval",	&_eval_list)
      || get(file, "function",	&_function_list)
      || get(file, "/*",	&dummy_c_comment)
      || get(file, "//",	&dummy_cxx_comment)
      ;
    paren -= file.skip1b("}");
    if (paren == 0) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Device)");
      break;
    }else if (file.stuck(&here)) {itested();
      file.warn(0, "bad Device");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Device::print(std::ostream& out)const
{
  out <<
    "device " << name() << " {\n"
    "  parse_name " << parse_name() << ";\n"
    "  id_letter " << id_letter() << ";\n"
    "  model_type " << model_type() << ";\n"
    "  circuit " << circuit() << "\n"
    "  tr_probe " << probes() << "\n"
    "  device " << device() << "\n"
    "  common " << common() << "\n"
    "  tr_eval {" << tr_eval() << "  }\n"
	      << eval_list()
	      << function_list() <<
    "}\n";
}
/*--------------------------------------------------------------------------*/
void C_Comment::parse(CS& file)
{
  int here = file.cursor();
  for (;;) {
    file.skipto1('*');
    if (file.pmatch("*/")) {
      break;  // done with comment
    }else if (file.stuck(&here)) {untested();
      file.warn(0, "unterminated C comment");
      break;
    }else{untested();
      file.skip();
    }
  }
}
/*--------------------------------------------------------------------------*/
void Cxx_Comment::parse(CS& file)
{
  int here = file.cursor();
  file.skipto1('\n');
  if (file.stuck(&here)) {untested();
    file.warn(0, "unterminated C++ comment");
  }else{
  }
}
/*--------------------------------------------------------------------------*/
void Port::parse(CS& file)
{
  file >> _name;
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "short_to",	&_short_to)
      || get(file, "short_if",	&_short_if)
      ;
    if (file.skip1b(";")) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Port)");
      break;
    }else if (file.stuck(&here)) {
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Port::print(std::ostream& out)const
{
  if (short_to() != "" || short_if() != "") {
    out << name() << " short_to=\"" << short_to() 
	<< "\" short_if=\"" << short_if() << "\";\n";
  }else{
    out << name() << "; ";
  }
}
/*--------------------------------------------------------------------------*/
void Head::parse(CS& file)
{
  int here = file.cursor();
  _begin = file.fullstring();
  for (;;) {
    file.skipto1('*');
    if (file.pmatch("*/")) {
      _end = file.tail();
      break;  // done with head
    }else if (file.stuck(&here)) {untested();
      file.warn(0, "unterminated head");
      break;
    }else{
      file.skip();
    }
  }
}
/*--------------------------------------------------------------------------*/
File::File(const std::string& file_name)
  :_name(file_name),
   _file(CS_FILE(), file_name)
{
  _file.set_exact();
  get(_file, "/*",		&_head);
  int here = _file.cursor();
  for (;;) {
    ONE_OF
      || get(_file, "/*",	 &dummy_c_comment)
      || get(_file, "//",	 &dummy_cxx_comment)
      || get(_file, "h_headers", &_h_headers)
      || get(_file, "cc_headers",&_cc_headers)
      || get(_file, "device",	 &_device_list)
      || get(_file, "model", 	 &_model_list)
      || get(_file, "h_direct",  &_h_direct)
      || get(_file, "cc_direct", &_cc_direct)
      ;
    if (!_file.more()) {
      break;
    }else if (_file.stuck(&here)) {itested();
      _file.warn(0, "syntax error, need head or model");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
